@@ -27,7 +27,7 @@ import XSLForms.Utils
 from XSLTools import XMLCalendar
 from WebCalendar.Common import rdfcalendar_item_types, rdfcalendar_calendar_item_types
 import os
-import time
+import time, datetime
 import urllib
 
 # Resource classes.
@@ -285,6 +285,7 @@ class ItemEditorResource(XSLForms.Resources.WebResources.XSLFormsResource):
         except IndexError:
             language = ""
         stylesheet_parameters["locale"] = language
+        stylesheet_parameters["root"] = attributes["root"]
 
         # Actually send the response.
 
@@ -332,6 +333,11 @@ class ItemEditorResource(XSLForms.Resources.WebResources.XSLFormsResource):
                     prepare_xsl_list = self.prepare_transform("card")
 
                 doc = self.get_result(prepare_xsl_list, doc)
+
+            # Convert the end dates to the form expected by the template.
+
+            self._fix_end_dates(doc, -1)
+
         else:
             doc = None
 
@@ -381,20 +387,35 @@ class ItemEditorResource(XSLForms.Resources.WebResources.XSLFormsResource):
                 if summary_value:
                     # NOTE: This will not work with various leading characters.
                     # Change the summary into a URL-encoded identifier.
-                    item = self.store.get_item_from_identifier(urllib.quote(summary_value))
+                    item = self.store.get_item_from_identifier(urllib.quote(summary_value).replace("/", "%2F").replace("#", "%23"))
                     while self.store.has_item(item):
                         summary_value += "'"
-                        item = self.store.get_item_from_identifier(urllib.quote(summary_value))
+                        item = self.store.get_item_from_identifier(urllib.quote(summary_value).replace("/", "%2F").replace("#", "%23"))
                 else:
                     return 0
             else:
                 return 0
 
+        # Convert the end dates from the form expected by the template.
+
+        self._fix_end_dates(doc, 1)
+
         # Add a new item.
 
-        self.store.add_item(item, doc)
+        self.store.add_item(item, item_type, doc)
         return 1
 
+    def _fix_end_dates(self, doc, days):
+
+        # Fix any end dates.
+
+        dtends = doc.xpath("/item/*/dtend")
+        for dtend in dtends:
+            end_date = dtend.getAttribute("datetime")
+            if end_date and len(end_date) == 8:
+                end_dt = datetime.datetime(int(end_date[:4]), int(end_date[4:6]), int(end_date[6:8]))
+                dtend.setAttribute("datetime", (end_dt + datetime.timedelta(days=days)).strftime("%Y%m%d"))
+
     # Methods adding information for saving.
 
     def _add_dates(self, doc, item):
@@ -527,12 +548,11 @@ class ItemEditorResource(XSLForms.Resources.WebResources.XSLFormsResource):
         """
 
         if value != "":
-            suggestion_items = self.store.get_filtered_items([
-                (["organizer", "uri"], ("startswith", value)),
-                (["attendee", "uri"], ("startswith", value)),
-                (["email", "details"], ("startswith", value)),
-                (["fn", "details"], ("startswith", value))
-                ])
+            suggestion_items = self.store.get_items_using_attributes({
+                "search-type" : "person-name",
+                "search-value" : "%" + value + "%", # NOTE: SQL-like syntax.
+                "search-operation" : "contains"
+                })
         else:
             suggestion_items = []
 
