@@ -26,7 +26,6 @@ import XSLForms.Resources.WebResources
 import XSLForms.Utils
 from XSLTools import XMLCalendar
 from WebCalendar.Exporter import export_items
-from WebCalendar.Common import default_item_types
 import os
 import time # for default filter information
 
@@ -66,6 +65,14 @@ class CalendarViewerResource(XSLForms.Resources.WebResources.XSLFormsResource):
 
         pvpath = trans.get_processed_virtual_path_info(self.path_encoding)
 
+        # Get the items in the form.
+
+        documents = form.get_documents()
+
+        if documents.has_key("items"):
+            items = documents["items"]
+            items_root = items.xpath("*")[0]
+
         # Find the date information.
 
         attributes = trans.get_attributes()
@@ -75,7 +82,7 @@ class CalendarViewerResource(XSLForms.Resources.WebResources.XSLFormsResource):
 
         # Find the items according to the attributes.
 
-        found_items = self.store.get_items_using_attributes(attributes, default_item_types)
+        found_items = self.store.get_items_using_attributes(attributes)
 
         # Check for download actions.
 
@@ -86,23 +93,9 @@ class CalendarViewerResource(XSLForms.Resources.WebResources.XSLFormsResource):
             export_items(found_items, self.store, stream)
             raise EndOfResponse
 
-        elif parameters.has_key("new-event"):
-            new_path = trans.update_path(pvpath, "../event/new/")
-            trans.redirect(trans.encode_path(new_path, self.path_encoding))
-            # Response ended!
-
-        elif parameters.has_key("new-to-do"):
-            new_path = trans.update_path(pvpath, "../to-do/new/")
-            trans.redirect(trans.encode_path(new_path, self.path_encoding))
-            # Response ended!
-
-        elif parameters.has_key("new-card"):
-            new_path = trans.update_path(pvpath, "../card/new/")
-            trans.redirect(trans.encode_path(new_path, self.path_encoding))
-            # Response ended!
-
-        elif parameters.has_key("new-journal"):
-            new_path = trans.update_path(pvpath, "../journal/new/")
+        elif parameters.has_key("new"):
+            new_type = items.xpath("items/new-type/@value")[0].nodeValue
+            new_path = trans.update_path(pvpath, "../%s/new/" % new_type)
             trans.redirect(trans.encode_path(new_path, self.path_encoding))
             # Response ended!
 
@@ -126,6 +119,10 @@ class CalendarViewerResource(XSLForms.Resources.WebResources.XSLFormsResource):
 
         self.store.fill_element(items, table, found_items)
 
+        # Add item types.
+
+        add_item_types(items, items_root, self.store)
+
         # Start the response.
 
         trans.set_content_type(ContentType("application/xhtml+xml", self.encoding))
@@ -157,6 +154,7 @@ class CalendarViewerResource(XSLForms.Resources.WebResources.XSLFormsResource):
         except IndexError:
             language = ""
         stylesheet_parameters["locale"] = language
+        stylesheet_parameters["root"] = attributes["root"]
 
         # Complete the response.
 
@@ -234,7 +232,7 @@ class ListViewerResource(XSLForms.Resources.WebResources.XSLFormsResource):
 
         # Filter according to supplied criteria.
 
-        found_items = self.store.get_items_using_attributes(attributes, default_item_types)
+        found_items = self.store.get_items_using_attributes(attributes)
 
         # Check for download actions or editing actions.
 
@@ -252,23 +250,9 @@ class ListViewerResource(XSLForms.Resources.WebResources.XSLFormsResource):
             export_items(selected_items, self.store, stream)
             raise EndOfResponse
 
-        elif parameters.has_key("new-event"):
-            new_path = trans.update_path(pvpath, "../event/new/")
-            trans.redirect(trans.encode_path(new_path, self.path_encoding))
-            # Response ended!
-
-        elif parameters.has_key("new-to-do"):
-            new_path = trans.update_path(pvpath, "../to-do/new/")
-            trans.redirect(trans.encode_path(new_path, self.path_encoding))
-            # Response ended!
-
-        elif parameters.has_key("new-card"):
-            new_path = trans.update_path(pvpath, "../card/new/")
-            trans.redirect(trans.encode_path(new_path, self.path_encoding))
-            # Response ended!
-
-        elif parameters.has_key("new-journal"):
-            new_path = trans.update_path(pvpath, "../journal/new/")
+        elif parameters.has_key("new"):
+            new_type = items.xpath("items/new-type/@value")[0].nodeValue
+            new_path = trans.update_path(pvpath, "../%s/new/" % new_type)
             trans.redirect(trans.encode_path(new_path, self.path_encoding))
             # Response ended!
 
@@ -283,6 +267,10 @@ class ListViewerResource(XSLForms.Resources.WebResources.XSLFormsResource):
 
         set_filter_attributes(items_root, attributes)
 
+        # Add item types.
+
+        add_item_types(items, items_root, self.store)
+
         # Produce the triple information.
 
         table = items.createElement("table")
@@ -320,6 +308,7 @@ class ListViewerResource(XSLForms.Resources.WebResources.XSLFormsResource):
         except IndexError:
             language = ""
         stylesheet_parameters["locale"] = language
+        stylesheet_parameters["root"] = attributes["root"]
 
         # Complete the response.
 
@@ -388,9 +377,6 @@ class PropertyViewerResource(XSLForms.Resources.WebResources.XSLFormsResource):
 
         else:
 
-            found_items = self.store.get_items_using_attributes(attributes, default_item_types)
-            set_filter_attributes(items_root, attributes)
-
             # Set special attributes in order to indicate different kinds of
             # resources in the container.
 
@@ -403,12 +389,17 @@ class PropertyViewerResource(XSLForms.Resources.WebResources.XSLFormsResource):
 
             if attributes.has_key("collection-export"):
                 items_root.setAttribute("leaf", "yes")
+                items_root.setAttribute("collection-export", "yes")
+                found_items = []
             else:
                 # Fix the path information, if necessary.
 
                 if not encoded_pvpath.endswith("/"):
                     encoded_pvpath += "/"
 
+                found_items = self.store.get_items_using_attributes(attributes)
+                set_filter_attributes(items_root, attributes)
+
         # Produce the triple information.
 
         table = items.createElement("table")
@@ -424,6 +415,7 @@ class PropertyViewerResource(XSLForms.Resources.WebResources.XSLFormsResource):
         stylesheet_parameters = {
             "path" : trans.get_path_without_info() + encoded_pvpath
             }
+        stylesheet_parameters["root"] = attributes["root"]
 
         # Ensure that an output stylesheet exists.
         # Use the list template.
@@ -451,6 +443,8 @@ class SimpleViewerResource(XSLForms.Resources.WebResources.XSLFormsResource):
         'form' information.
         """
 
+        attributes = trans.get_attributes()
+
         # Get the processed path details and a version suitable for output.
 
         pvpath = trans.get_processed_virtual_path_info(self.path_encoding)
@@ -478,6 +472,7 @@ class SimpleViewerResource(XSLForms.Resources.WebResources.XSLFormsResource):
         stylesheet_parameters = {
             "path" : trans.get_path_without_info() + encoded_pvpath
             }
+        stylesheet_parameters["root"] = attributes["root"]
 
         # Ensure that an output stylesheet exists.
 
@@ -511,21 +506,52 @@ class RootViewerResource(SimpleViewerResource):
         }
     default_document = "root"
 
+    def __init__(self, store):
+        self.store = store
+
     def get_directories(self, trans, pvpath):
-        d = ["all", "event", "journal", "message", "to-do"]
-        if pvpath == "/":
-            d.append("calendar")
-            d.append("card")
+        d = ["all"]
+        d += self.store.get_supported_item_types()
+        #if pvpath == "/":
+        #    d.append("calendar")
+        #    d.append("card")
         if not trans.get_attributes().has_key("filter-type"):
             d.append("related-to")
             d.append("person")
         return d
 
     def get_files(self, trans, pvpath):
-        return ["all.ics", "event.ics", "to-do.ics"]
+        return ["all.ics", "event.ics", "to-do.ics", "free-busy.ics"]
 
 # Common functions.
 
+def add_item_types(items, items_root, store):
+
+    """
+    Add the item types to the 'items' document, under 'items_root', using
+    information from the 'store'.
+    """
+
+    new_type = items.createElement("new-type")
+    items_root.appendChild(new_type)
+    item_types = items.createElement("item-types")
+    items_root.appendChild(item_types)
+
+    item_type = items.createElement("item-type")
+    item_type.setAttribute("value", "all")
+    item_types.appendChild(item_type)
+
+    supported_item_types = store.get_supported_item_types()
+    supported_item_types.sort()
+
+    for item_type_label in supported_item_types:
+        new_type_enum = items.createElement("new-type-enum")
+        new_type_enum.setAttribute("value", item_type_label)
+        new_type.appendChild(new_type_enum)
+        item_type = items.createElement("item-type")
+        item_type.setAttribute("value", item_type_label)
+        item_types.appendChild(item_type)
+
 def set_filter_attributes(element, attributes):
 
     "In the given 'element', set filter information from the 'attributes'."
