@@ -3,7 +3,7 @@
 """
 A Web calendar application.
 
-Copyright (C) 2005, 2006 Paul Boddie <paul@boddie.org.uk>
+Copyright (C) 2005, 2006, 2007 Paul Boddie <paul@boddie.org.uk>
 
 This software is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License as
@@ -21,13 +21,14 @@ If not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 """
 
-__version__ = "0.1.4"
+__version__ = "0.2.1"
 
 import os
 
 # Site map imports.
 
 from WebStack.Resources.ResourceMap import MapResource
+from WebStack.Resources.Selectors import PathSelector
 from WebStack.Resources.Static import DirectoryResource
 from WebCalendar.Viewer import *
 from WebCalendar.Selector import *
@@ -35,6 +36,10 @@ from WebCalendar.Editor import *
 from WebCalendar.Exporter import *
 from WebCalendar.Importer import *
 
+# Important constants.
+
+encoding = "utf-8"
+
 # Site map initialisation.
 
 def get_site(store):
@@ -50,6 +55,7 @@ def get_site(store):
     item_importer_resource = ItemImporterResource(store)
     item_exporter_resource = ItemExporterResource(store)
     collection_exporter_resource = CollectionExporterResource(store)
+    freebusy_exporter_resource = FreeBusyExporterResource(store)
 
     # Set up file serving and some directory indexes.
 
@@ -57,7 +63,7 @@ def get_site(store):
     styles_dir = DirectoryResource(os.path.join(directory, "styles"), {"css" : "text/css"})
     scripts_dir = DirectoryResource(os.path.join(directory, "scripts"), {"js" : "text/javascript"})
     images_dir = DirectoryResource(os.path.join(directory, "images"), {"png" : "image/png", "svg" : "image/svg+xml"})
-    webdav_root = RootViewerResource()
+    webdav_root = RootViewerResource(store)
     webdav_months = MonthViewerResource()
 
     # Here's what the site looks like:
@@ -72,6 +78,8 @@ def get_site(store):
                                                                   | | /<filename>
                                                                   | |
                                                                   | -> ExporterSelectorResource
+                                                                  | |
+                                                                  | -> (FreeBusyExporterResource)
                                                                   |
                                                                   | /
                                                                   |
@@ -84,11 +92,8 @@ def get_site(store):
     # Make a simple Web site.
 
     view_choices = MapResource({
-        "styles" : styles_dir,
-        "scripts" : scripts_dir,
-        "images" : images_dir,
         "" : ViewSelectorResource(list_viewer_resource, calendar_viewer_resource, prop_viewer_resource)
-        }) # not pass_through: identifiers are removed from the vpath
+        }, path_encoding=encoding) # not pass_through: identifiers are removed from the vpath
 
     # Handle access to the above where identifiers are present.
     # (This also happens when styles and scripts are being accessed through an
@@ -98,35 +103,43 @@ def get_site(store):
         item_exporter_resource, prop_viewer_resource)
 
     item_choices = MapResource({
-        # Item selectors...
-        "event" : TypeSelectorResource(view_choices),
-        "to-do" : TypeSelectorResource(view_choices),
-        "card" : TypeSelectorResource(view_choices),
-        "journal" : TypeSelectorResource(view_choices),
-        "message" : TypeSelectorResource(view_choices),
-        "calendar" : TypeSelectorResource(view_choices),
         # Exporters...
         "event.ics" : ExporterSelectorResource("event", collection_exporter_resource, prop_viewer_resource),
         "to-do.ics" : ExporterSelectorResource("to-do", collection_exporter_resource, prop_viewer_resource),
         "all.ics" : ExporterSelectorResource("all", collection_exporter_resource, prop_viewer_resource),
+        "free-busy.ics" : ExporterSelectorResource("free-busy", freebusy_exporter_resource, prop_viewer_resource),
         # Views...
         "all" : view_choices,
         "" : TopSelectorResource("all/", webdav_root)
-        }, pass_through=1)
+        }, pass_through=1, path_encoding=encoding)
+
+    # Add type-based handlers, dependent on what the store offers.
+
+    for item_type_name in store.get_supported_item_types():
+        item_choices.mapping[item_type_name] = TypeSelectorResource(view_choices)
 
     filter_choices = MapResource({
         # URI selectors producing views...
         "related-to" : FilterSelectorResource("related-to", item_choices, prop_viewer_resource),
         "person" : FilterSelectorResource("person", item_choices, prop_viewer_resource, "card"),
+        # Pass through...
         None : item_choices
-        }, pass_through=1)
+        }, pass_through=1, path_encoding=encoding)
 
     date_choices = MapResource({
         "date" : DateSelectorResource(filter_choices, webdav_months),
+        # Pass through...
         None : filter_choices
-        }, pass_through=1)
+        }, pass_through=1, path_encoding=encoding)
 
-    return date_choices
+    root_choices = MapResource({
+        "styles" : styles_dir,
+        "scripts" : scripts_dir,
+        "images" : images_dir,
+        None : date_choices
+        }, pass_through=1, path_encoding=encoding)
+
+    return PathSelector(root_choices)
 
 # Resource preparation ahead of time - useful for making installations.
 
