@@ -19,5 +19,7 @@ sys.path.insert(0, os.path.join(base, "applications"))
 
 import WebCalendar
 WebCalendar.prepare_resources()
+import WebCalendarPortal
+WebCalendarPortal.prepare_resources()
 
 # vim: tabstop=4 expandtab shiftwidth=4
